<!--
pyp5js
Copyright (C) 2019-2021 Bernardo Fontes

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->
<!DOCTYPE html>

<!-- pyp5js index.html boilerplate -->
<html lang="">
  <head>

    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{{ sketch_name }} - pyp5js (using pyodide)</title>
    <style> body, html, canvas {padding: 0; margin: 0; overflow: hidden;} </style>

    <script src="{{ pyodide_js_url }}"></script>
    <script src="{{ p5_js_url }}"></script>
    <script src="{{ sketch_js_url }}"  type="module"></script>
  </head>

  <body>
    <div id="sketch-holder">
          <!-- You sketch will go here! -->
    </div>
  </body>
</html>
